@@ -15,6 +15,7 @@ import (
 // Op represents an IPP Operation Code
 type Op Code
 
+// Op codes
 const (
 	OpPrintJob             Op = 0x0002 // Print-Job: Print a single file
 	OpPrintURI             Op = 0x0003 // Print-URI: Print a single URL
@@ -143,236 +144,128 @@ const (
 
 // String() returns a Status name, as defined by RFC 8010
 func (op Op) String() string {
-	switch op {
-	case OpPrintJob:
-		return "Print-Job"
-	case OpPrintURI:
-		return "Print-URI"
-	case OpValidateJob:
-		return "Validate-Job"
-	case OpCreateJob:
-		return "Create-Job"
-	case OpSendDocument:
-		return "Send-Document"
-	case OpSendURI:
-		return "Send-URI"
-	case OpCancelJob:
-		return "Cancel-Job"
-	case OpGetJobAttributes:
-		return "Get-Job-Attribute"
-	case OpGetJobs:
-		return "Get-Jobs"
-	case OpGetPrinterAttributes:
-		return "Get-Printer-Attributes"
-	case OpHoldJob:
-		return "Hold-Job"
-	case OpReleaseJob:
-		return "Release-Job"
-	case OpRestartJob:
-		return "Restart-Job"
-	case OpPausePrinter:
-		return "Pause-Printer"
-	case OpResumePrinter:
-		return "Resume-Printer"
-	case OpPurgeJobs:
-		return "Purge-Jobs"
-	case OpSetPrinterAttributes:
-		return "Set-Printer-Attributes"
-	case OpSetJobAttributes:
-		return "Set-Job-Attributes"
-	case OpGetPrinterSupportedValues:
-		return "Get-Printer-Supported-Values"
-	case OpCreatePrinterSubscriptions:
-		return "Create-Printer-Subscriptions"
-	case OpCreateJobSubscriptions:
-		return "Create-Job-Subscriptions"
-	case OpGetSubscriptionAttributes:
-		return "Get-Subscription-Attributes"
-	case OpGetSubscriptions:
-		return "Get-Subscriptions"
-	case OpRenewSubscription:
-		return "Renew-Subscription"
-	case OpCancelSubscription:
-		return "Cancel-Subscription"
-	case OpGetNotifications:
-		return "Get-Notifications"
-	case OpSendNotifications:
-		return "Send-Notifications"
-	case OpGetResourceAttributes:
-		return "Get-Resource-Attributes"
-	case OpGetResourceData:
-		return "Get-Resource-Data"
-	case OpGetResources:
-		return "Get-Resources"
-	case OpGetPrintSupportFiles:
-		return "Get-Printer-Support-Files"
-	case OpEnablePrinter:
-		return "Enable-Printer"
-	case OpDisablePrinter:
-		return "Disable-Printer"
-	case OpPausePrinterAfterCurrentJob:
-		return "Pause-Printer-After-Current-Job"
-	case OpHoldNewJobs:
-		return "Hold-New-Jobs"
-	case OpReleaseHeldNewJobs:
-		return "Release-Held-New-Jobs"
-	case OpDeactivatePrinter:
-		return "Deactivate-Printer"
-	case OpActivatePrinter:
-		return "Activate-Printer"
-	case OpRestartPrinter:
-		return "Restart-Printer"
-	case OpShutdownPrinter:
-		return "Shutdown-Printer"
-	case OpStartupPrinter:
-		return "Startup-Printer"
-	case OpReprocessJob:
-		return "Reprocess-Job"
-	case OpCancelCurrentJob:
-		return "Cancel-Current-Job"
-	case OpSuspendCurrentJob:
-		return "Suspend-Current-Job"
-	case OpResumeJob:
-		return "Resume-Job"
-	case OpPromoteJob:
-		return "Promote-Job"
-	case OpScheduleJobAfter:
-		return "Schedule-Job-After"
-	case OpCancelDocument:
-		return "Cancel-Document"
-	case OpGetDocumentAttributes:
-		return "Get-Document-Attributes"
-	case OpGetDocuments:
-		return "Get-Documents"
-	case OpDeleteDocument:
-		return "Delete-Document"
-	case OpSetDocumentAttributes:
-		return "Set-Document-Attributes"
-	case OpCancelJobs:
-		return "Cancel-Jobs"
-	case OpCancelMyJobs:
-		return "Cancel-My-Jobs"
-	case OpResubmitJob:
-		return "Resubmit-Job"
-	case OpCloseJob:
-		return "Close-Job"
-	case OpIdentifyPrinter:
-		return "Identify-Printer"
-	case OpValidateDocument:
-		return "Validate-Document"
-	case OpAddDocumentImages:
-		return "Add-Document-Images"
-	case OpAcknowledgeDocument:
-		return "Acknowledge-Document"
-	case OpAcknowledgeIdentifyPrinter:
-		return "Acknowledge-Identify-Printer"
-	case OpAcknowledgeJob:
-		return "Acknowledge-Job"
-	case OpFetchDocument:
-		return "Fetch-Document"
-	case OpFetchJob:
-		return "Fetch-Job"
-	case OpGetOutputDeviceAttributes:
-		return "Get-Output-Device-Attributes"
-	case OpUpdateActiveJobs:
-		return "Update-Active-Jobs"
-	case OpDeregisterOutputDevice:
-		return "Deregister-Output-Device"
-	case OpUpdateDocumentStatus:
-		return "Update-Document-Status"
-	case OpUpdateJobStatus:
-		return "Update-Job-Status"
-	case OpupdateOutputDeviceAttributes:
-		return "Update-Output-Device-Attributes"
-	case OpGetNextDocumentData:
-		return "Get-Next-Document-Data"
-	case OpAllocatePrinterResources:
-		return "Allocate-Printer-Resources"
-	case OpCreatePrinter:
-		return "Create-Printer"
-	case OpDeallocatePrinterResources:
-		return "Deallocate-Printer-Resources"
-	case OpDeletePrinter:
-		return "Delete-Printer"
-	case OpGetPrinters:
-		return "Get-Printers"
-	case OpShutdownOnePrinter:
-		return "Shutdown-One-Printer"
-	case OpStartupOnePrinter:
-		return "Startup-One-Printer"
-	case OpCancelResource:
-		return "Cancel-Resource"
-	case OpCreateResource:
-		return "Create-Resource"
-	case OpInstallResource:
-		return "Install-Resource"
-	case OpSendResourceData:
-		return "Send-Resource-Data"
-	case OpSetResourceAttributes:
-		return "Set-Resource-Attributes"
-	case OpCreateResourceSubscriptions:
-		return "Create-Resource-Subscriptions"
-	case OpCreateSystemSubscriptions:
-		return "Create-System-Subscriptions"
-	case OpDisableAllPrinters:
-		return "Disable-All-Printers"
-	case OpEnableAllPrinters:
-		return "Enable-All-Printers"
-	case OpGetSystemAttributes:
-		return "Get-System-Attributes"
-	case OpGetSystemSupportedValues:
-		return "Get-System-Supported-Values"
-	case OpPauseAllPrinters:
-		return "Pause-All-Printers"
-	case OpPauseAllPrintersAfterCurrentJob:
-		return "Pause-All-Printers-After-Current-Job"
-	case OpRegisterOutputDevice:
-		return "Register-Output-Device"
-	case OpRestartSystem:
-		return "Restart-System"
-	case OpResumeAllPrinters:
-		return "Resume-All-Printers"
-	case OpSetSystemAttributes:
-		return "Set-System-Attributes"
-	case OpShutdownAllPrinters:
-		return "Shutdown-All-Printers"
-	case OpStartupAllPrinters:
-		return "Startup-All-Printers"
-	case OpCupsGetDefault:
-		return "CUPS-Get-Default"
-	case OpCupsGetPrinters:
-		return "CUPS-Get-Printers"
-	case OpCupsAddModifyPrinter:
-		return "CUPS-Add-Modify-Printer"
-	case OpCupsDeletePrinter:
-		return "CUPS-Delete-Printer"
-	case OpCupsGetClasses:
-		return "CUPS-Get-Classes"
-	case OpCupsAddModifyClass:
-		return "CUPS-Add-Modify-Class"
-	case OpCupsDeleteClass:
-		return "CUPS-Delete-Class"
-	case OpCupsAcceptJobs:
-		return "CUPS-Accept-Jobs"
-	case OpCupsRejectJobs:
-		return "CUPS-Reject-Jobs"
-	case OpCupsSetDefault:
-		return "CUPS-Set-Default"
-	case OpCupsGetDevices:
-		return "CUPS-Get-Devices"
-	case OpCupsGetPpds:
-		return "CUPS-Get-PPDs"
-	case OpCupsMoveJob:
-		return "CUPS-Move-Job"
-	case OpCupsAuthenticateJob:
-		return "CUPS-Authenticate-Job"
-	case OpCupsGetPpd:
-		return "CUPS-Get-PPD"
-	case OpCupsGetDocument:
-		return "CUPS-Get-Document"
-	case OpCupsCreateLocalPrinter:
-		return "CUPS-Create-Local-Printer"
+	if int(op) < len(opNames) {
+		if s := opNames[op]; s != "" {
+			return s
+		}
 	}
 
 	return fmt.Sprintf("0x%4.4x", int(op))
 }
+
+var opNames = [...]string{
+	OpPrintJob:                        "Print-Job",
+	OpPrintURI:                        "Print-URI",
+	OpValidateJob:                     "Validate-Job",
+	OpCreateJob:                       "Create-Job",
+	OpSendDocument:                    "Send-Document",
+	OpSendURI:                         "Send-URI",
+	OpCancelJob:                       "Cancel-Job",
+	OpGetJobAttributes:                "Get-Job-Attribute",
+	OpGetJobs:                         "Get-Jobs",
+	OpGetPrinterAttributes:            "Get-Printer-Attributes",
+	OpHoldJob:                         "Hold-Job",
+	OpReleaseJob:                      "Release-Job",
+	OpRestartJob:                      "Restart-Job",
+	OpPausePrinter:                    "Pause-Printer",
+	OpResumePrinter:                   "Resume-Printer",
+	OpPurgeJobs:                       "Purge-Jobs",
+	OpSetPrinterAttributes:            "Set-Printer-Attributes",
+	OpSetJobAttributes:                "Set-Job-Attributes",
+	OpGetPrinterSupportedValues:       "Get-Printer-Supported-Values",
+	OpCreatePrinterSubscriptions:      "Create-Printer-Subscriptions",
+	OpCreateJobSubscriptions:          "Create-Job-Subscriptions",
+	OpGetSubscriptionAttributes:       "Get-Subscription-Attributes",
+	OpGetSubscriptions:                "Get-Subscriptions",
+	OpRenewSubscription:               "Renew-Subscription",
+	OpCancelSubscription:              "Cancel-Subscription",
+	OpGetNotifications:                "Get-Notifications",
+	OpSendNotifications:               "Send-Notifications",
+	OpGetResourceAttributes:           "Get-Resource-Attributes",
+	OpGetResourceData:                 "Get-Resource-Data",
+	OpGetResources:                    "Get-Resources",
+	OpGetPrintSupportFiles:            "Get-Printer-Support-Files",
+	OpEnablePrinter:                   "Enable-Printer",
+	OpDisablePrinter:                  "Disable-Printer",
+	OpPausePrinterAfterCurrentJob:     "Pause-Printer-After-Current-Job",
+	OpHoldNewJobs:                     "Hold-New-Jobs",
+	OpReleaseHeldNewJobs:              "Release-Held-New-Jobs",
+	OpDeactivatePrinter:               "Deactivate-Printer",
+	OpActivatePrinter:                 "Activate-Printer",
+	OpRestartPrinter:                  "Restart-Printer",
+	OpShutdownPrinter:                 "Shutdown-Printer",
+	OpStartupPrinter:                  "Startup-Printer",
+	OpReprocessJob:                    "Reprocess-Job",
+	OpCancelCurrentJob:                "Cancel-Current-Job",
+	OpSuspendCurrentJob:               "Suspend-Current-Job",
+	OpResumeJob:                       "Resume-Job",
+	OpPromoteJob:                      "Promote-Job",
+	OpScheduleJobAfter:                "Schedule-Job-After",
+	OpCancelDocument:                  "Cancel-Document",
+	OpGetDocumentAttributes:           "Get-Document-Attributes",
+	OpGetDocuments:                    "Get-Documents",
+	OpDeleteDocument:                  "Delete-Document",
+	OpSetDocumentAttributes:           "Set-Document-Attributes",
+	OpCancelJobs:                      "Cancel-Jobs",
+	OpCancelMyJobs:                    "Cancel-My-Jobs",
+	OpResubmitJob:                     "Resubmit-Job",
+	OpCloseJob:                        "Close-Job",
+	OpIdentifyPrinter:                 "Identify-Printer",
+	OpValidateDocument:                "Validate-Document",
+	OpAddDocumentImages:               "Add-Document-Images",
+	OpAcknowledgeDocument:             "Acknowledge-Document",
+	OpAcknowledgeIdentifyPrinter:      "Acknowledge-Identify-Printer",
+	OpAcknowledgeJob:                  "Acknowledge-Job",
+	OpFetchDocument:                   "Fetch-Document",
+	OpFetchJob:                        "Fetch-Job",
+	OpGetOutputDeviceAttributes:       "Get-Output-Device-Attributes",
+	OpUpdateActiveJobs:                "Update-Active-Jobs",
+	OpDeregisterOutputDevice:          "Deregister-Output-Device",
+	OpUpdateDocumentStatus:            "Update-Document-Status",
+	OpUpdateJobStatus:                 "Update-Job-Status",
+	OpupdateOutputDeviceAttributes:    "Update-Output-Device-Attributes",
+	OpGetNextDocumentData:             "Get-Next-Document-Data",
+	OpAllocatePrinterResources:        "Allocate-Printer-Resources",
+	OpCreatePrinter:                   "Create-Printer",
+	OpDeallocatePrinterResources:      "Deallocate-Printer-Resources",
+	OpDeletePrinter:                   "Delete-Printer",
+	OpGetPrinters:                     "Get-Printers",
+	OpShutdownOnePrinter:              "Shutdown-One-Printer",
+	OpStartupOnePrinter:               "Startup-One-Printer",
+	OpCancelResource:                  "Cancel-Resource",
+	OpCreateResource:                  "Create-Resource",
+	OpInstallResource:                 "Install-Resource",
+	OpSendResourceData:                "Send-Resource-Data",
+	OpSetResourceAttributes:           "Set-Resource-Attributes",
+	OpCreateResourceSubscriptions:     "Create-Resource-Subscriptions",
+	OpCreateSystemSubscriptions:       "Create-System-Subscriptions",
+	OpDisableAllPrinters:              "Disable-All-Printers",
+	OpEnableAllPrinters:               "Enable-All-Printers",
+	OpGetSystemAttributes:             "Get-System-Attributes",
+	OpGetSystemSupportedValues:        "Get-System-Supported-Values",
+	OpPauseAllPrinters:                "Pause-All-Printers",
+	OpPauseAllPrintersAfterCurrentJob: "Pause-All-Printers-After-Current-Job",
+	OpRegisterOutputDevice:            "Register-Output-Device",
+	OpRestartSystem:                   "Restart-System",
+	OpResumeAllPrinters:               "Resume-All-Printers",
+	OpSetSystemAttributes:             "Set-System-Attributes",
+	OpShutdownAllPrinters:             "Shutdown-All-Printers",
+	OpStartupAllPrinters:              "Startup-All-Printers",
+	OpCupsGetDefault:                  "CUPS-Get-Default",
+	OpCupsGetPrinters:                 "CUPS-Get-Printers",
+	OpCupsAddModifyPrinter:            "CUPS-Add-Modify-Printer",
+	OpCupsDeletePrinter:               "CUPS-Delete-Printer",
+	OpCupsGetClasses:                  "CUPS-Get-Classes",
+	OpCupsAddModifyClass:              "CUPS-Add-Modify-Class",
+	OpCupsDeleteClass:                 "CUPS-Delete-Class",
+	OpCupsAcceptJobs:                  "CUPS-Accept-Jobs",
+	OpCupsRejectJobs:                  "CUPS-Reject-Jobs",
+	OpCupsSetDefault:                  "CUPS-Set-Default",
+	OpCupsGetDevices:                  "CUPS-Get-Devices",
+	OpCupsGetPpds:                     "CUPS-Get-PPDs",
+	OpCupsMoveJob:                     "CUPS-Move-Job",
+	OpCupsAuthenticateJob:             "CUPS-Authenticate-Job",
+	OpCupsGetPpd:                      "CUPS-Get-PPD",
+	OpCupsGetDocument:                 "CUPS-Get-Document",
+	OpCupsCreateLocalPrinter:          "CUPS-Create-Local-Printer",
+}
